unit TB2Skin;

{

  Toolbar2000 Modifications (patch.txt) &
              Skin+ Implementation (TB2Skin.Pas)
  Copyright 2001 by Mihail Haralabos. All rights reserved.
  Version 1.00 Beta 8

  * Toolbar 2.0.4 Release Compatible *

  * This code may be used and modified by anyone so long as
    this header and copyright  information remains intact and
    ONLY with use with Toolbar 2000 Component *

  Contributors:
  ColorDarker, ColorLighter functions (C) Aaron Chan

  Floating Windows OfficeXP & WindowsXP Style (C) Andreas Holstenson
  Windows XP Shadows (C) Andreas Holstenson

  Skin Manager, Docked Captions, Component Additions & Changes (C) Dean Harmon
  Other color conversions found somewhere on Internet.

}

interface

Uses
 Windows, Messages, Controls, SysUtils, Classes, Graphics;

Const
 TBSkinVersion = 'Version 1.00 Beta 8';
 XPShadowMargin = 4;
 ShadowContrast = 50;

type
  TRGBQuad =
    PACKED RECORD
      rgbBlue : BYTE;
      rgbGreen: BYTE;
      rgbRed  : BYTE;
      rgbReserved:  BYTE
    END;
  pRGBQuad = ^TRGBQuad;

  TRGBQuadArray = ARRAY[WORD] OF TRGBQuad;
  pRGBQuadArray = ^TRGBQuadArray;

  TClipRange = Record
   Start: Integer;
   Finish: Integer;
  End;

  ESkinError = class(Exception);
  TTBBaseSkin = class;
  PNotifyEvent = ^TNotifyEvent;

  TTBSkins = (tbsDisabled, tbsOfficeXP, tbsWindowsXP);
  TTBColorSets =(tbcOfficeXP, tbcWindowsXP, tbcCustom);
  TTBSkinColor = (cFace, cPopup, cBorder, cToolbar, cDockBorderIn,
                  cDockBorderOut, cDragHandle, cCheckBox, cImageList,
                  cImgListShadow, cSelBar, cSelBarBorder, cSelItem,
                  cSelItemBorder, cSelPushed, cSelPushedBorder,
                  cChecked, cCheckedOver, cSeparator);
  TTBOption  = (tboPopupOverlap, tboShadow, tboDockedCaptions,
                tboMenuTBColor, tboGradSelItem);
  TTBGradSel = (tbgLeftRight, tbgTopBottom);
  TShadowStyle = (sOfficeXP, sWindowsXP);
  TTBOptions  = Set Of TTBOption;

  TTBColors = Class(TPersistent)
   private
    FParent: TTBBaseSkin;

    FFace,
    FPopup,
    FBorder,
    FToolbar,
    FCheckBox,
    FImageList,
    FDragHandle,
    FChecked,
    FCheckedOver,
    FDockBorderIn,
    FDockBorderOut,
    FSelBar,
    FSelBarBorder,
    FSelItem,
    FSelItemBorder,
    FSelPushed,
    FSelPushedBorder,
    FImgListShadow,
    FSeparator,
    FGradStart,
    FGradEnd           : TColor;

    rgbFace,
    rgbPopup,
    rgbBorder,
    rgbToolbar,
    rgbCheckBox,
    rgbImageList,
    rgbDragHandle,
    rgbChecked,
    rgbCheckedOver,
    rgbDockBorderIn,
    rgbDockBorderOut,
    rgbSelBar,
    rgbSelBarBorder,
    rgbSelItem,
    rgbSelItemBorder,
    rgbSelPushed,
    rgbSelPushedBorder,
    rgbSeparator,
    rgbImgListShadow   : TColor;

    procedure SetFace(Const Value: TColor);
    procedure SetPopup(Const Value: TColor);
    procedure SetBorder(Const Value: TColor);
    procedure SetToolbar(Const Value: TColor);
    procedure SetCheckBox(Const Value: TColor);
    procedure SetImageList(Const Value: TColor);
    procedure SetDockBorderIn(Const Value: TColor);
    procedure SetDockBorderOut(Const Value: TColor);
    procedure SetDragHandle(Const Value: TColor);
    procedure SetChecked(Const Value: TColor);
    procedure SetCheckedOver(Const Value: TColor);
    procedure SetSelBar(Const Value: TColor);
    procedure SetSelBarBorder(Const Value: TColor);
    procedure SetSelItem(Const Value: TColor);
    procedure SetSelItemBorder(Const Value: TColor);
    procedure SetSelPushed(Const Value: TColor);
    procedure SetSelPushedBorder(Const Value: TColor);
    procedure SetImgListShadow(Const Value: TColor);
    procedure SetSeparator(Const Value: TColor);
   public
    constructor Create(AOwner: TTBBaseSkin);
   published
    property tcFace: TColor read FFace write SetFace;
    property tcPopup: TColor read FPopup write SetPopup;
    property tcBorder: TColor read FBorder write SetBorder;
    property tcToolbar: TColor read FToolbar write SetToolbar;
    property tcCheckBox: TColor read FCheckBox write SetCheckBox;
    property tcImageList: TColor read FImageList write SetImageList;
    property tcDockBorderIn: TColor read FDockBorderIn write SetDockBorderIn;
    property tcDockBorderOut: TColor read FDockBorderOut write SetDockBorderOut;
    property tcDragHandle: TColor read FDragHandle write SetDragHandle;
    property tcChecked: TColor read FChecked write SetChecked;
    property tcCheckedOver: TColor read FCheckedOver write SetCheckedOver;
    property tcSelBar: TColor read FSelBar write SetSelBar;
    property tcSelBarBorder: TColor read FSelBarBorder write SetSelBarBorder;
    property tcSelItem: TColor read FSelItem write SetSelItem;
    property tcSelItemBorder: TColor read FSelItemBorder write SetSelItemBorder;
    property tcSelPushed: TColor read FSelPushed write SetSelPushed;
    property tcSelPushedBorder: TColor read FSelPushedBorder write SetSelPushedBorder;
    property tcSeparator: TColor read FSeparator write SetSeparator;
    property tcImgListShadow: TColor read FImgListShadow write SetImgListShadow;
    property tcGradStart: TColor read FGradStart write FGradStart;
    property tcGradEnd: TColor read FGradEnd write FGradEnd;
   end;

  TTBSkinManager = class(TList)
  private
    FOriginalDefault : TTBBaseSkin;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure SetSkinAsDefault(ASkin: TTBBaseSkin);
    function AddSkin(ASkin: TTBBaseSkin): Integer;
    procedure RemoveSkin(ASkin: TTBBaseSkin);
  end;

  TTBBaseSkin = class(TComponent)
  private
    FVersion: String;
    FTBColors: TTBColors;
    FNotifyList: TList;
    FTBSkin: TTBSkins;
    FTBColorSet: TTBColorSets;
    FTBOptions: TTBOptions;
    FTBGradSel: TTBGradSel;
    FIsDefault: Boolean;
    FShadowStyle: TShadowStyle;

    FViews: TList;
    FDockables: TList;

    procedure SetTBSkin(Const Value: TTBSkins);
    procedure SetTBColorSet(Const Value: TTBColorSets);
    procedure SetDefault(Const Value: Boolean);
    procedure SetOptions(const Value: TTBOptions);

    procedure OfficeXPColorSet;
    procedure WindowsXPColorSet;

    function GetComponentVersion: String;
  protected
    procedure RegisterChanges (Proc: TNotifyEvent); virtual;
    procedure UnregisterChanges (Proc: TNotifyEvent); virtual;
    procedure AssignColorsToRGB; virtual;

    procedure RepaintRegisteredClasses; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure CalculateColorSet;
    function RGBColor(Const AColor: TTBSkinColor) : TColorRef;
    procedure Assign(Source: TPersistent); override;
    function GetPopupNCSize: Integer;

    property SkinType: TTBSkins read FTBSkin write SetTBSkin default tbsOfficeXP;
    property ColorSet: TTBColorSets read FTBColorSet write SetTBColorSet default tbcOfficeXP;
    property Colors: TTBColors read FTBColors write FTBColors;
    property Options: TTBOptions read FTBOptions write SetOptions default [tboShadow];
    property IsDefaultSkin: Boolean read FIsDefault write SetDefault;
    property Gradient: TTBGradSel read FTBGradSel write FTBGradSel;
    property ShadowStyle: TShadowStyle read FShadowStyle write FShadowStyle default sOfficeXP;

    property Version: String read GetComponentVersion write FVersion;

    function RegisterDockable(ADockable: TObject): Integer;
    function UnregisterDockable(ADockable: TObject): Boolean;

    procedure FillGradient(Const DC: HDC; Const ARect: TRect);
  end;

  TTBSkin = class(TTBBaseSkin)
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ColorSet; //Must be before Colors!
    property Colors;
    property IsDefaultSkin;
    property Options;
    property Gradient;
    property ShadowStyle;
    property SkinType;
    property Version;
  end;

{- Shadow Related -}

  TShadow = class(TWinControl)
  private
    FRight: Boolean;
    FParent: TWinControl;
    FCorner1: Boolean;
    FCorner2: Boolean;
    FShadowStyle: TShadowStyle;

    MemDC: HDC;
    MemBitmap: HBitmap;

    procedure OfficeXPShadow(Const DC: HDC);
    procedure WindowsXPShadow(Const DC: HDC);

    procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    FPrepared: Boolean;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    ClipRange: TClipRange;

    constructor Create(AOwner: TComponent; AParent: TWinControl;
                       ARight: Boolean); reintroduce;
    destructor Destroy; override;

    procedure Prepare;
    procedure UnPrepare;
  published
    property Corner1: Boolean read FCorner1 write FCorner1;
    property Corner2: Boolean read FCorner2 write FCorner2;
    property ShadowStyle: TShadowStyle read FShadowStyle write FShadowStyle;
  end;

function IsWindowsXP: Boolean;
function IsWindowsXPThemed: Boolean;

var
  DefaultSkin : TTBBaseSkin;
  TBSkinManager: TTBSkinManager;

implementation

Uses
  Registry, TB2Common, TB2Dock;

{ Helper Functions }

procedure HSVtoRGB(Const Hue, Saturation, Value: Integer; var Red, Green, Blue: Integer);
var
  P, Q, R: Integer;
  S: Single;
begin
  if Saturation = 0 then
  begin
    Red := Value;
    Green := Value;
    Blue := Value;
  end else begin
    S := Saturation * Frac(Hue / 60);
    P := Round(Value * (255 - Saturation) / 256);
    Q := Round(Value * (255 - S) / 256);
    R := Round(Value * (255 - Saturation + S) / 256);
    case Hue div 60 of
      0: begin
           Red := Value;
           Green := R;
           Blue := P;
         end;
      1: begin
           Red := Q;
           Green := Value;
           Blue := P;
         end;
      2: begin
           Red := P;
           Green := Value;
           Blue := P;
         end;
      3: begin
           Red := P;
           Green := Q;
           Blue := Value;
         end;
      4: begin
           Red := R;
           Green := P;
           Blue := Value;
         end;
      5: begin
           Red := Value;
           Green := P;
           Blue := Q;
         end;
    end; //case
  end; //end
end;

procedure RGBtoHSV(Const Red, Green, Blue: Integer; var Hue, Saturation, Value: Integer);
var
  Maximum, Minimum: Integer;
  Rc, Gc, Bc: Single;
  H: Single;
begin
  Maximum := Max(Red, Max(Green, Blue));
  Minimum := Min(Red, Min(Green, Blue));
  Value := Maximum;
  if Maximum <> 0 then
    Saturation := MulDiv(Maximum - Minimum, 255, Maximum)
  else
    Saturation := 0;
  if Saturation = 0 then
    Hue := 0 // arbitrary value
  else
  begin
    Assert(Maximum <> Minimum);
    Rc := (Maximum - Red) / (Maximum - Minimum);
    Gc := (Maximum - Green) / (Maximum - Minimum);
    Bc := (Maximum - Blue) / (Maximum - Minimum);
    if Red = Maximum then
      H := Bc - Gc
    else if Green = Maximum then
      H := 2 + Rc - Bc
    else
    begin
      Assert(Blue = Maximum);
      H := 4 + Gc - Rc;
    end;
    H := H * 60;
    if H < 0 then
      H := H + 360;
    Hue := Round(H);
  end;
end;

function HSVtoColor(Const Hue, Saturation, Value: Integer): TColor;
var
  Red, Green, Blue: Integer;
begin
  HSVtoRGB(Hue, Saturation, Value, Red, Green, Blue);
  Result := RGB(Red, Green, Blue);
end;

procedure ColortoHSV(Const Color: TColor; var Hue, Saturation, Value: Integer);
var
  RGB: LongWord;
begin
  RGB := ColorToRGB(Color);
  RGBtoHSV(GetRValue(RGB), GetGValue(RGB), GetBValue(RGB), Hue, Saturation, Value);
end;

Function ColorLighter(Const Color: TColor; Const Percent: Byte):TColor;
Var
  R, G, B: Byte;
  FColor: TColorRef;
begin
  FColor := ColorToRGB(Color);

  R := GetRValue(FColor);
  G := GetGValue(FColor);
  B := GetBValue(FColor);

  R := R + muldiv(255 -r, Percent, 100);
  G := G + muldiv(255 -g, Percent, 100);
  B := B + muldiv(255 -b, Percent, 100);

  Result := TColor(RGB(R, G, B));
end;

Function ColorDarker(Const OriginalColor: TColor; Const Percent: Byte): TColor;
Var
  R, G, B: Integer;
  WinColor: Integer;
begin
  WinColor := ColorToRGB(OriginalColor);

  R := GetRValue(WinColor);
  G := GetGValue(WinColor);
  B := GetBValue(WinColor);

  R := R - Percent;
  G := G - Percent;
  B := B - Percent;

  If R < 0 Then R := 0;
  If G < 0 Then G := 0;
  If B < 0 Then B := 0;

  Result := TColor(RGB(R, G, B));
end;

Function ColorLighterRGB(Const Color: TColorRef; Const Percent: Byte): TColorRef;
Var
  R, G, B: Byte;
begin
  R := GetRValue(Color);
  G := GetGValue(Color);
  B := GetBValue(Color);

  R := R + muldiv(255 -r, Percent, 100);
  G := G + muldiv(255 -g, Percent, 100);
  B := B + muldiv(255 -b, Percent, 100);

  Result := RGB(R, G, B);
end;

Function ColorDarkerRGB(Const Color: TColorRef; Const Percent: Byte): TColorRef;
Var
  R, G, B: Integer;
begin
  R := GetRValue(Color);
  G := GetGValue(Color);
  B := GetBValue(Color);

  R := R - Percent;
  G := G - Percent;
  B := B - Percent;

  If R < 0 Then R := 0;
  If G < 0 Then G := 0;
  If B < 0 Then B := 0;

  Result := RGB(R, G, B);
end;

function BlendColors(const color1, color2: TColor; amount: Extended): TColor;
var
  R,R2,G,G2,B,B2: Integer;
  win1, win2: Integer;
begin
  win1 := ColorToRGB(color1);
  win2 := ColorToRGB(color2);

  // Convert first
  R := GetRValue(win1);
  G := GetGValue(win1);
  B := GetBValue(win1);
  // Convert second
  R2 := GetRValue(win2);
  G2 := GetGValue(win2);
  B2 := GetBValue(win2);

  // Mix them into color1

  b2:=round((1-amount)*b+amount*b2);
  g2:=round((1-amount)*g+amount*g2);
  r2:=round((1-amount)*r+amount*r2);

  // Just to make sure...
  if R2 < 0 then R2 := 0;
  if G2 < 0 then G2 := 0;
  if B2 < 0 then B2 := 0;

  // If we ever reach white we should revert to first color
  if R2 > 255 then R2 := r;
  if G2 > 255 then G2 := r;
  if B2 > 255 then B2 := r;

  Result := TColor(RGB(R2, G2, B2));
end;

function IsWindowsXP: Boolean;
begin
 If (Win32Platform = VER_PLATFORM_WIN32_NT) And
    ((Win32MajorVersion > 5) Or
     ((Win32MajorVersion = 5) And
      (Win32MinorVersion >= 1))) Then
  Result := True
 Else
  Result := False;
end;

function IsWindowsXPThemed: Boolean;
Var
 Reg: TRegistry;
begin
 Result := False;

 If Not IsWindowsXP Then
  Exit;

 Reg := TRegistry.Create;
 Reg.RootKey := HKEY_CURRENT_USER;

 If (Reg.OpenKey('Software\Microsoft\Windows\CurrentVersion\ThemeManager', False)) And
    (Reg.ReadString('ThemeActive') = '1') Then
  Result := True;

 Reg.Free;
end;

{ TTBColors }

constructor TTBColors.Create(AOwner: TTBBaseSkin);
begin
  Inherited Create;

  FParent := AOwner;
end;

procedure TTBColors.SetFace(Const Value: TColor);
begin
  If Not (FFace = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FFace := Value;
      rgbFace := ColorToRGB(FFace);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcFace := FFace;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetPopup(Const Value: TColor);
begin
  If Not (FPopup = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FPopup := Value;
      rgbPopup := ColorToRGB(FPopup);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcPopup := FPopup;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetCheckBox(Const Value: TColor);
begin
  If Not (FCheckBox = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FCheckBox := Value;
      rgbCheckBox := ColorToRGB(FCheckBox);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcCheckBox := FCheckBox;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelBar(Const Value: TColor);
begin
  If Not (FSelBar = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelBar := Value;
      rgbSelBar := ColorToRGB(FSelBar);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelBar := FSelBar;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelBarBorder(Const Value: TColor);
begin
  If Not (FSelBarBorder = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelBarBorder := Value;
      rgbSelBarBorder := ColorToRGB(FSelBarBorder);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelBarBorder := FSelBarBorder;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelItem(Const Value: TColor);
begin
  If Not (FSelItem = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelItem := Value;
      rgbSelItem := ColorToRGB(FSelItem);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelItem := FSelItem;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelItemBorder(Const Value: TColor);
begin
  If Not (FSelItemBorder = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelItemBorder := Value;
      rgbSelItemBorder := ColorToRGB(FSelItemBorder);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelItemBorder := FSelItemBorder;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetImageList(Const Value: TColor);
begin
  If Not (FImageList = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FImageList := Value;
      rgbImageList := ColorToRGB(FImageList);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcImageList := FImageList;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetBorder(Const Value: TColor);
begin
  If Not (FBorder = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FBorder := Value;
      rgbBorder := ColorToRGB(FBorder);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcBorder := FBorder;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetDockBorderIn(Const Value: TColor);
begin
 If Not (FDockBorderIn = Value) Then
  If FParent.FTBColorSet = tbcCustom Then
   Begin
    FDockBorderIn := Value;
    rgbDockBorderIn := ColorToRGB(FDockBorderIn);

    If FParent.FIsDefault then
     Begin
      DefaultSkin.FTBColors.tcDockBorderIn := FDockBorderIn;
      DefaultSkin.RepaintRegisteredClasses;
     End
    Else
     FParent.RepaintRegisteredClasses;
   End;
end;

procedure TTBColors.SetDockBorderOut(Const Value: TColor);
begin
 If Not (FDockBorderOut = Value) Then
  If FParent.FTBColorSet = tbcCustom Then
   Begin
    FDockBorderOut := Value;
    rgbDockBorderOut := ColorToRGB(FDockBorderOut);

    If FParent.FIsDefault then
     Begin
      DefaultSkin.FTBColors.tcDockBorderOut := FDockBorderOut;
      DefaultSkin.RepaintRegisteredClasses;
     End
    Else
     FParent.RepaintRegisteredClasses;
   End;
end;

procedure TTBColors.SetDragHandle(Const Value: TColor);
begin
  If Not (FDragHandle = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FDragHandle := Value;
      rgbDragHandle := ColorToRGB(FDragHandle);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcDragHandle := FDragHandle;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetChecked(Const Value: TColor);
begin
  If Not (FChecked = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FChecked := Value;
      rgbChecked := ColorToRGB(FChecked);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcChecked := FChecked;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetCheckedOver(Const Value: TColor);
begin
  If Not (FCheckedOver = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FCheckedOver := Value;
      rgbCheckedOver := ColorToRGB(FCheckedOver);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcCheckedOver := FCheckedOver;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetToolbar(Const Value: TColor);
begin
  If Not (FToolbar = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FToolbar := Value;
      rgbToolbar := ColorToRGB(FToolbar);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcToolbar := FToolbar;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelPushed(Const Value: TColor);
begin
  If Not (FSelPushed = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelPushed := Value;
      rgbSelPushed := ColorToRGB(FSelPushed);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelPushed := FSelPushed;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSelPushedBorder(Const Value: TColor);
begin
  If Not (FSelPushedBorder = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSelPushedBorder := Value;
      rgbSelPushedBorder := ColorToRGB(FSelPushedBorder);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSelPushedBorder := FSelPushedBorder;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetSeparator(Const Value: TColor);
begin
  If Not (FSeparator = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FSeparator := Value;
      rgbSeparator := ColorToRGB(FSeparator);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcSeparator := FSeparator;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

procedure TTBColors.SetImgListShadow(Const Value: TColor);
begin
  If Not (FImgListShadow = Value) Then
  Begin
    If FParent.FTBColorSet = tbcCustom Then
    begin
      FImgListShadow := Value;
      rgbImgListShadow := ColorToRGB(FImgListShadow);

      If FParent.FIsDefault then
       Begin
        DefaultSkin.FTBColors.tcImgListShadow := FImgListShadow;
        DefaultSkin.RepaintRegisteredClasses;
       End
      Else
       FParent.RepaintRegisteredClasses;
    end;
  End;
end;

{ TTBSkinManager }

function TTBSkinManager.AddSkin(ASkin: TTBBaseSkin): Integer;
Var
 Counter: Integer;
begin
 For Counter := 0 To Count - 1 Do
  If ASkin = Self[Counter] Then
   Begin
    Result := Counter;
    exit;
   End;

 Result := Add(ASkin);

 //If it's the first skin that is being added,
 //make it the default instead

 If Count = 1 Then
  Begin
   DefaultSkin.Assign(ASkin);
   ASkin.FIsDefault := True;
  End;
end;

constructor TTBSkinManager.Create;
begin
  FOriginalDefault := TTBBaseSkin.Create(nil);
  FOriginalDefault.Assign(DefaultSkin);
end;

destructor TTBSkinManager.Destroy;
begin
  DefaultSkin.Assign(FOriginalDefault);
  FOriginalDefault.Free;
  Inherited;
end;

procedure TTBSkinManager.RemoveSkin(ASkin: TTBBaseSkin);
var i : Integer;
    WasDefault : Boolean;
begin
  i := IndexOf(ASkin);

  if i <> -1 then
  begin
    WasDefault := ASkin.IsDefaultSkin;
    Delete(i);
    if WasDefault and (Count > 0) then
    begin
      TTBBaseSkin(Self[0]).FIsDefault := True;
    end else if (Count = 0) then begin
      DefaultSkin.Assign(FOriginalDefault);
    end;
  end else begin
    Raise ESkinError.Create('The skin cannot be removed because it''s not part of the SkinManager');
  end;
end;

procedure TTBSkinManager.SetSkinAsDefault(ASkin: TTBBaseSkin);
VAR i, j : Integer;
begin

  if Assigned(ASkin) then
  begin
    i := IndexOf(ASkin);

    if (i <> -1) and ASkin.IsDefaultSkin then
    begin
      //make sure all the others are *not* the default anymore
      for j := 0 to Count - 1 do
        if TTBSkin(self[j]) <> ASkin then
          TTBSkin(self[j]).FIsDefault := False;
    end;

    if i = -1 then
    begin
      raise ESkinError.Create('This skin has not been added to the SkinManager.');
      exit;
    end;

    ASkin.FIsDefault := True;
    DefaultSkin.Assign(ASkin);
  end else begin
    //if nil is passed, then we set it to the original
    DefaultSkin.Assign(FOriginalDefault);
  end;
end;

{ TTBBaseSkin }

procedure TTBBaseSkin.OfficeXPColorSet;
Var
  Color1,
  Color2,
  Color3: Integer;
begin
 With FTBColors Do
  Begin
   FFace := clBtnFace;
   FBorder := clBtnShadow;
   FToolbar := ColorLighter(clBtnFace, 19);
   FCheckBox := ColorLighter(clHighlight, 70);
   FDragHandle := ColorLighter(clBtnShadow, 24);

   FSelBar := ColorLighter(clHighlight, 70);
   FSelBarBorder := clHighlight;

   FSelItem := FSelBar;
   FSelItemBorder  := clHighlight;

   FSelPushed := ColorLighter(clHighlight, 50);
   FSelPushedBorder := clHighlight;

   FImageList := ColorLighter(clBtnFace, 19);
   FImgListShadow := ColorLighter(clHighlight, 45);

   ColortoHSV(clWindow, Color1, Color2, Color3);
   Inc(Color1, 30);
   Inc(Color2, 7);
   FPopup := HSVToColor(Color1, Color2, Color3);

   FChecked := ColorLighter(FSelBar, 30);
   FCheckedOver := FSelPushed;

   FDockBorderIn := ColorDarker(clBtnShadow, 20);
   FDockBorderOut := FDockBorderIn;

   FGradStart := FSelItem;
   FGradEnd := clWhite;

   FSeparator := ColorLighter(clBtnShadow, 20);
  End;

 AssignColorsToRGB;
end;

procedure TTBBaseSkin.WindowsXPColorSet;
begin
 With FTBColors Do
  Begin
   FFace := clWindow;
   FPopup := clWindow;
   FBorder := clBtnShadow;
   FToolbar := clBtnFace;
   FDragHandle := clBtnShadow;

   FSelBar := clWindow;
   FSelBarBorder := clBtnShadow;

   FSelItem := ColorLighter(clHighlight, 13);
   FSelItemBorder := clHighlight;

   FSelPushed := ColorDarker(FSelItem, 20);
   FSelPushedBorder := FSelItem;

   FImageList := clWindow;
   FImgListShadow := clBtnShadow;

   FCheckBox := FSelItem;
   FChecked := FSelBar;
   FCheckedOver := FSelPushed;

   FDockBorderIn := FPopup;
   FDockBorderOut := clBlack;

   FGradStart := FSelItem;
   FGradEnd := clWhite;

   FSeparator := FBorder;
  End;

 AssignColorsToRGB;
end;

constructor TTBBaseSkin.Create(AOwner: TComponent);
begin
 Inherited Create(AOwner);

 FTBColors := TTBColors.Create(Self);
 FTBSkin := tbsOfficeXP;
 FTBOptions := [tboShadow];
 FShadowStyle := sOfficeXP;

 CalculateColorSet;

 FIsDefault := False;

 FViews := TList.Create;
 FDockables := TList.Create;
end;

destructor TTBBaseSkin.Destroy;
Var
  Counter: Integer;
begin
  Destroying;

  Inherited;

  If Assigned(FNotifyList) Then
  Begin
    For Counter := FNotifyList.Count-1 DownTo 0 Do
      Dispose(PNotifyEvent(FNotifyList[Counter]));

    FNotifyList.Free;
  End;

  FViews.Free;
  FDockables.Free;
  FTBColors.Free;
end;

procedure TTBBaseSkin.RegisterChanges(Proc: TNotifyEvent);
var
  I: Integer;
  P: PNotifyEvent;
begin
  if FNotifyList = nil then
    FNotifyList := TList.Create;
  for I := 0 to FNotifyList.Count-1 do begin
    P := FNotifyList[I];
    if (TMethod(P^).Code = TMethod(Proc).Code) and
       (TMethod(P^).Data = TMethod(Proc).Data) then
      Exit;
  end;
  FNotifyList.Expand;
  New (P);
  P^ := Proc;
  FNotifyList.Add (P);
end;

procedure TTBBaseSkin.UnregisterChanges (Proc: TNotifyEvent);
var
  I: Integer;
  P: PNotifyEvent;
begin
  if FNotifyList = nil then
    Exit;
  for I := 0 to FNotifyList.Count-1 do begin
    P := FNotifyList[I];
    if (TMethod(P^).Code = TMethod(Proc).Code) and
       (TMethod(P^).Data = TMethod(Proc).Data) then begin
      FNotifyList.Delete (I);
      Dispose (P);
      Break;
    end;
  end;
end;

procedure TTBBaseSkin.CalculateColorSet;
begin
 Case FTBColorSet Of
    tbcOfficeXP: OfficeXPColorSet;
   tbcWindowsXP: WindowsXPColorSet;
  End;
end;

procedure TTBBaseSkin.SetTBSkin(Const Value: TTBSkins);
begin
  If Not (FTBSkin = Value) Then
  Begin
    FTBSkin := Value;

    Case FTBSkin Of
      tbsOfficeXP: Begin
                    ColorSet := tbcOfficeXP;
                    Options := Options + [tboShadow] - [tboPopupOverlap];
                    ShadowStyle := sOfficeXP;
                    CalculateColorSet;
                   End;
     tbsWindowsXP: Begin
                    ColorSet := tbcWindowsXP;
                    Options := Options + [tboShadow, tboPopupOverlap];
                    ShadowStyle := sWindowsXP;
                    CalculateColorSet;
                   End;
    End;

    if FIsDefault then
    begin
      DefaultSkin.SkinType := FTBSkin;
      DefaultSkin.RepaintRegisteredClasses;
    end;

    RepaintRegisteredClasses;
  End;
end;

procedure TTBBaseSkin.SetTBColorSet(Const Value: TTBColorSets);
begin
 If Not (FTBSkin = tbsDisabled) Then
  If Not (FTBColorSet = Value) Then
   Begin
    FTBColorSet := Value;
    CalculateColorSet;

    If FIsDefault Then
     Begin
      DefaultSkin.ColorSet := FTBColorSet;
      DefaultSkin.RepaintRegisteredClasses;
     End;

    RepaintRegisteredClasses;
   End;
end;

function TTBBaseSkin.RGBColor(Const AColor: TTBSkinColor): TColorRef;
begin
 With FTBColors Do
  Case AColor Of
   cFace : Result := rgbFace;
   cPopup : Result := rgbPopup;
   cBorder : Result := rgbBorder;
   cToolbar : Result := rgbToolbar;
   cCheckBox : Result := rgbCheckBox;
   cImageList : Result := rgbImageList;
   cDragHandle : Result := rgbDragHandle;
   cChecked: Result := rgbChecked;
   cCheckedOver : Result := rgbCheckedOver;
   cDockBorderIn : Result := rgbDockBorderIn;
   cDockBorderOut : Result := rgbDockBorderOut;
   cSelBar : Result := rgbSelBar;
   cSelBarBorder : Result := rgbSelBarBorder;
   cSelItem : Result := rgbSelItem;
   cSelItemBorder : Result := rgbSelItemBorder;
   cSelPushed : Result := rgbSelPushed;
   cSelPushedBorder : Result := rgbSelPushedBorder;
   cSeparator: Result := rgbSeparator;
   cImgListShadow : Result := rgbImgListShadow;
   Else Result := ColorToRGB(clWhite);//No result
  End;
end;

procedure TTBBaseSkin.AssignColorsToRGB;
Var
 DC: HDC;
begin
 DC := GetDC(0);

 With FTBColors Do
  Begin
   rgbFace := GetNearestColor(DC, ColorToRGB(FFace));
   rgbBorder := GetNearestColor(DC, ColorToRGB(FBorder));
   rgbToolbar := GetNearestColor(DC, ColorToRGB(FToolbar));
   rgbCheckBox := GetNearestColor(DC, ColorToRGB(FCheckBox));
   rgbImageList := GetNearestColor(DC, ColorToRGB(FImageList));
   rgbDragHandle := GetNearestColor(DC, ColorToRGB(FDragHandle));
   rgbChecked := GetNearestColor(DC, ColorToRGB(FChecked));
   rgbCheckedOver := GetNearestColor(DC, ColorToRGB(FCheckedOver));
   rgbDockBorderIn := GetNearestColor(DC, ColorToRGB(FDockBorderIn));
   rgbDockBorderOut := GetNearestColor(DC, ColorToRGB(FDockBorderOut));
   rgbSelBar := GetNearestColor(DC, ColorToRGB(FSelBar));
   rgbSelBarBorder := GetNearestColor(DC, ColorToRGB(FSelBarBorder));
   rgbSelItem := GetNearestColor(DC, ColorToRGB(FSelItem));
   rgbSelItemBorder := GetNearestColor(DC, ColorToRGB(FSelItemBorder));
   rgbSelPushed := GetNearestColor(DC, ColorToRGB(FSelPushed));
   rgbSelPushedBorder := GetNearestColor(DC, ColorToRGB(FSelPushedBorder));
   rgbSeparator := GetNearestColor(DC, ColorToRGB(FSeparator));
   rgbImgListShadow := GetNearestColor(DC, ColorToRGB(FImgListShadow));
   rgbPopup := GetNearestColor(DC, ColorToRGB(FPopup));
  End;

 ReleaseDC(0, DC);
end;

procedure TTBBaseSkin.SetDefault(const Value: Boolean);
begin
 If Not (FIsDefault = Value) Then
  Begin
   FIsDefault := Value;

   If Value Then
    TBSkinManager.SetSkinAsDefault(Self)
   Else
    TBSkinManager.SetSkinAsDefault(Nil);

   DefaultSkin.RepaintRegisteredClasses;

   If Not FIsDefault Then
    RepaintRegisteredClasses;
  End;
end;

procedure TTBBaseSkin.Assign(Source: TPersistent);
begin
 With FTBColors Do
  If Source is TTBBaseSkin Then
   Begin
    SkinType := TTBBaseSkin(Source).SkinType;
    ColorSet := TTBBaseSkin(Source).ColorSet;
    tcFace := TTBBaseSkin(Source).FTBColors.tcFace;
    tcPopup := TTBBaseSkin(Source).FTBColors.tcPopup;
    tcBorder := TTBBaseSkin(Source).FTBColors.tcBorder;
    tcCheckBox := TTBBaseSkin(Source).FTBColors.tcCheckBox;
    tcDragHandle := TTBBaseSkin(Source).FTBColors.tcDragHandle;
    tcChecked := TTBBaseSkin(Source).FTBColors.tcChecked;
    tcCheckedOver := TTBBaseSkin(Source).FTBColors.tcCheckedOver;
    tcDockBorderIn := TTBBaseSkin(Source).FTBColors.tcDockBorderIn;
    tcDockBorderOut := TTBBaseSkin(Source).FTBColors.tcDockBorderOut;
    tcSelBar := TTBBaseSkin(Source).FTBColors.tcSelBar;
    tcSelBarBorder := TTBBaseSkin(Source).FTBColors.tcSelBarBorder;
    tcSelItem := TTBBaseSkin(Source).FTBColors.tcSelItem;
    tcSelItemBorder := TTBBaseSkin(Source).FTBColors.tcSelItemBorder;
    tcSelPushed := TTBBaseSkin(Source).FTBColors.tcSelPushed;
    tcSelPushedBorder := TTBBaseSkin(Source).FTBColors.tcSelPushedBorder;
    tcImageList := TTBBaseSkin(Source).FTBColors.tcImageList;
    tcImgListShadow := TTBBaseSkin(Source).FTBColors.tcImgListShadow;

    Options := TTBBaseSkin(Source).Options;
   End
  Else
   Inherited;
end;

function TTBBaseSkin.RegisterDockable(ADockable: TObject): Integer;
var i : Integer;
begin
  i := FDockables.IndexOf(ADockable);
  if i = -1 then
  begin
    Result := FDockables.Add(ADockable);
  end else Result := i;
end;

function TTBBaseSkin.UnregisterDockable(ADockable: TObject): Boolean;
var i : Integer;
begin
  i := FDockables.IndexOf(ADockable);
  if i <> -1 then
  begin
    FDockables.Delete(i);
    Result := True
  end else Result := False;
end;

procedure TTBBaseSkin.RepaintRegisteredClasses;
Var
 Counter: Integer;
begin
 For Counter := 0 To FDockables.Count - 1 Do
  If TTBCustomDockableWindow(FDockables[Counter]).HandleAllocated Then
   If TTBCustomDockableWindow(FDockables[Counter]).Docked Then
    SetWindowPos(TTBCustomDockableWindow(FDockables[Counter]).Handle,
                 0, 0, 0, 0, 0,
                 SWP_FRAMECHANGED or SWP_NOACTIVATE or
                 SWP_NOMOVE or SWP_NOZORDER)
   Else
    With TTBCustomDockableWindow(FDockables[Counter]) Do
     Begin
      TTBCustomDockableWindow(FDockables[Counter]).Perform(WM_SETREDRAW, 0, 0);
      SetBounds(Left, Top, Width, Height -1);
      SetBounds(Left, Top, Width, Height +1);
      TTBCustomDockableWindow(FDockables[Counter]).Perform(WM_SETREDRAW, 1, 0);

      SetWindowPos(TTBCustomDockableWindow(FDockables[Counter]).Handle,
                   0, 0, 0, 0, 0,
                   SWP_FRAMECHANGED or SWP_NOACTIVATE or
                   SWP_NOMOVE or SWP_NOZORDER);
     End;
end;

procedure TTBBaseSkin.SetOptions(const Value: TTBOptions);
begin
  If FTBOptions <> Value Then
  Begin
    FTBOptions := Value;

    If FIsDefault Then
     Begin
      DefaultSkin.Options := FTBOptions;
      DefaultSkin.RepaintRegisteredClasses;
     End
    Else
     RepaintRegisteredClasses;
  End;
end;

function TTBBaseSkin.GetPopupNCSize: Integer;
begin
 If (FTBSkin = tbsDisabled)  Or
    (FTBSkin = tbsWindowsXP) Then
  Result := 3
 Else
  Result := 2;
end;

function TTBBaseSkin.GetComponentVersion;
begin
 Result := TBSkinVersion;
end;

procedure TTBBaseSkin.FillGradient(Const DC: HDC; Const ARect: TRect);
Var
 cs,
 ce,
 rc1,
 rc2,
 gc1,
 gc2,
 bc1,
 bc2: Integer;
 Counter: Integer;
 Brush: HBrush;
begin
 cs := FTBColors.FGradStart;
 ce := FTBColors.FGradEnd;

 rc1 := GetRValue(cs);
 gc1 := GetGValue(cs);
 bc1 := GetBValue(cs);
 rc2 := GetRValue(ce);
 gc2 := GetGValue(ce);
 bc2 := GetBValue(ce);

 If FTBGradSel = tbgLeftRight Then
  For Counter := ARect.Top to ARect.Bottom Do
   Begin
    Brush := CreateSolidBrush(
     RGB((rc1 + (((rc2 - rc1) * (ARect.Top + Counter)) div ARect.Bottom)),
                  (gc1 + (((gc2 - gc1) * (ARect.Top + Counter)) div ARect.Bottom)),
                  (bc1 + (((bc2 - bc1) * (ARect.Top + Counter)) div ARect.Bottom))));
    FillRect(DC, Rect(0, ARect.Top, ARect.Right, ARect.Bottom - Counter + 1), Brush);
    DeleteObject(Brush);
   End
 Else
  For Counter := ARect.Left to ARect.Right Do
   Begin
    Brush := CreateSolidBrush(
     RGB((rc1 + (((rc2 - rc1) * (ARect.Left + Counter)) div ARect.Right)),
                  (gc1 + (((gc2 - gc1) * (ARect.Left + Counter)) div ARect.Right)),
                  (bc1 + (((bc2 - bc1) * (ARect.Left + Counter)) div ARect.Right))));
    FillRect(DC, Rect(ARect.Left, ARect.Top, ARect.Right - Counter +1, ARect.Bottom), Brush);
    DeleteObject(Brush);
   End;
end;

{ TTBSkin }

constructor TTBSkin.Create(AOwner: TComponent);
begin
  Inherited;
  TBSkinManager.AddSkin(Self);
end;

destructor TTBSkin.Destroy;
begin
  Destroying;

  Inherited;

  If Assigned(TBSkinManager) then
   TBSkinManager.RemoveSkin(Self);
end;

{ TShadow }

constructor TShadow.Create(AOwner: TComponent; AParent: TWinControl;
                           ARight: Boolean);
begin
  Inherited Create(AOwner);

  FParent := AParent;
  FRight := ARight;
  ParentWindow := AParent.Handle;

  Corner1 := False;
  Corner2 := False;

  ClipRange.Start  := -1;
  ClipRange.Finish := 0;

  Hide;
end;

procedure TShadow.CreateParams(var Params: TCreateParams);
begin
 Inherited CreateParams(Params);

 with Params do begin
   Style := Style and not WS_CHILD or WS_POPUP;
   WindowClass.Style := WindowClass.Style or CS_SAVEBITS or
                        CS_PARENTDC or CS_BYTEALIGNWINDOW or
                        (not (CS_HREDRAW) and (CS_VREDRAW));
 end;
end;

destructor TShadow.Destroy;
begin
  If FPrepared Then
   UnPrepare;

  Inherited Destroy;
end;

procedure TShadow.WMNCPaint(var Message: TMessage);
begin
 Message.Result := 0;
end;

procedure TShadow.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
 BitBlt(Message.DC, 0, 0, Width, Height,
        MemDC, 0, 0, SRCCOPY);

 Message.Result := 0;
end;

procedure TShadow.OfficeXPShadow(Const DC: HDC);
Var
 X,
 Y: Integer;
begin
 If FRight Then
  For X := 0 To Width - 1 Do
   Begin
    //Draw The Main Shadow...
    For Y := Width To (Width * 2) - 1 Do
     If Not (Y in [ClipRange.Start..ClipRange.Finish]) then
      SetPixel(MemDC, X, Y, ColorDarkerRGB(GetPixel(MemDC, X, Y), 40 - ((X + ((Width * 2) - 1 - Y)) * 5)));

    //Draw Fade Out At Top Edge Shadow
    If Corner1 Then
     For Y := (Width * 2) To Height - 1 - (Width - 4) Do
      If Not (Y in [ClipRange.Start..ClipRange.Finish]) then
       SetPixel(MemDC, X, Y, ColorDarkerRGB(GetPixel(MemDC, X, Y), 50 - (X * 10)));
   End
 Else
  For Y := 0 To Height - 1 Do
   Begin
    //Draw Fade In At Left Edge Shadow
    If Corner1 Then
     For X := 0 To Height - 1 Do
      If Not (X in [ClipRange.Start..ClipRange.Finish]) then
       SetPixel(MemDC, X, Y, ColorDarkerRGB(GetPixel(MemDC, X, Y), 60 - ((Y + ((Height * 2) - 1 - X)) * 5)));

    //Draw The Main Shadow...
    For X := (Height) To Width - 1 - (Height) Do
     If Not (X in [ClipRange.Start..ClipRange.Finish]) then
      SetPixel(MemDC, X, Y, ColorDarkerRGB(GetPixel(MemDC, X, Y), 50 - (Y * 10)));

    //Draw Fade Out At Right Edge Shadow
    If Corner2 Then
     For X := 0 To Height -1 Do
      If Not (X in [ClipRange.Start..ClipRange.Finish]) then
       SetPixel(MemDC, Width-1-(X), Y, ColorDarkerRGB(GetPixel(MemDC, Width-1-(X), Y), ((X + ((Height * 2) - 1 - Y)) * 5)-15 ));
   End;
end;

procedure TShadow.WindowsXPShadow(Const DC: HDC);
Var
 X,
 Y: Integer;
 ColorToUse: TColor;
begin
 ColorToUse := TColor(RGB(30,30,30));

 If FRight Then
  For X := 0 To Width - 1 Do
   Begin
    For Y := Width To (Width * 2) - 1 Do
     If not (Y in [ClipRange.Start..ClipRange.Finish]) then
      SetPixel(MemDC, X, Y, BlendColors(GetPixel(MemDC, X, Y), ColorToUse, (40 - ((X + ((Width * 2) - 1 - Y)) * 15))/100));

    If Corner1 Then
     For Y := (Width * 2) To Height - 1 - (Width - 4) Do
      If not (Y in [ClipRange.Start..ClipRange.Finish]) then
       SetPixel(MemDC, X, Y, BlendColors(GetPixel(MemDC, X, Y), ColorToUse, (50 - (X * 15))/100));
   End
 Else
  For Y := 0 To Height - 1 Do
   Begin
    // Draw fade in at left edge shadow
    If Corner1 Then
     For X := 0 To Height - 1 Do
      If not (X in [ClipRange.Start..ClipRange.Finish]) then
       SetPixel(MemDC, X, Y, BlendColors(GetPixel(MemDC, X, Y), ColorToUse, (60 - ((Y + ((Height * 2) - 1 - X)) * 7.1))/100));

    // draw the main shadow...
    For X := (Height) To Width - 1 - (Height) Do
     If not (X in [ClipRange.Start..ClipRange.Finish]) then
      SetPixel(MemDC, X, Y, BlendColors(GetPixel(MemDC, X, Y), ColorToUse, ( 50 - (Y * 15))/100));

    // Draw fade out at right edge shadow (ah Note: I think it should be a straight shadow like _|)
    If Corner2 Then
     For X := 0 To Height -1 Do
      If not (X in [ClipRange.Start..ClipRange.Finish]) then
       if (X = 3) and (Y = 0) then
         SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, (50 - (0 * 15))/100))
       else
       if (X = 3) and (Y = 1) then
        SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (1 * 15))/100))
       else
       if (X = 2) and (Y < 2) then
        SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (1 * 15))/100))
       else
       if (X > 1) and (Y = 2) then
         SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (2 * 15))/100))
       else
       if (X = 1) and (Y < 3) then
         SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (2 * 15))/100))
       else
       if (X > 0) and (Y = 3) then
        SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (3 * 15))/100))
       else
       if (X = 0) and (Y < 4) then
        SetPixel(MemDC, Width-1-(X), Y, BlendColors(GetPixel(MemDC, Width-1-(X), Y), ColorToUse, ( 50 - (3 * 15))/100))
   End;
end;

procedure TShadow.Prepare;
Var
  DC: HDC;
begin
  If FPrepared Then
   Exit;

  MemDC := GetDC(0);
  MemBitmap := CreateCompatibleBitmap(MemDC, Width, Height);
  ReleaseDC(0, MemDC);

  MemDC := CreateCompatibleDC(0);
  SelectObject(MemDC, MemBitmap);

  ProcessPaintMessages;

  DC := GetDC(0);
  BitBlt(MemDC, 0, 0, Width, Height, DC,
         Left, Top, SRCCOPY);
  ReleaseDC(0, DC);

  If ShadowStyle = sOfficeXP Then
   OfficeXPShadow(DC)
  Else
   WindowsXPShadow(DC);

  FPrepared := True;
end;

procedure TShadow.UnPrepare;
begin
 If FPrepared Then
  Begin
   DeleteDC(MemDC);
   DeleteObject(MemBitmap);
  End;
end;

initialization
  DefaultSkin := TTBBaseSkin.Create(Nil);
  DefaultSkin.FTBSkin := tbsDisabled;
  DefaultSkin.Options := [];

  TBSkinManager := TTBSkinManager.Create;
finalization
  TBSkinManager.Free;
  TBSkinManager := nil;

  DefaultSkin.Free;
  DefaultSkin := nil;
end.

